/* ************************************************************************

   Copyright:

   License:

   Authors:

************************************************************************ */

/* ************************************************************************

#resource(image:image)

************************************************************************ */


var plasma_entities = new Array(
    {
        name      : "Plasma frequency",
        dimension : "frequency",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getPlasmaFrequency()"
    },
    {
        name      : "Gyrofrequency",
        dimension : "frequency",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getGyroFrequency(this.B)"
    },

    {
        name      : "Debye length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getDebyeLength()"
    },
    {
        name      : "Full Debye length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "this.getFullDebyeLength(%s1)"
    },
    {
        name      : "deBroglie length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getDeBroglieLength()"
    },
    {
        name      : "Mean interparticle distance",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getMeanDistance()"
    },
    {
        name      : "Distance of minimal approach",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getMinApproachDistance()"
    },
    {
        name      : "Gyroradius",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getGyroRadius(this.B)"
    },
    {
        name      : "Inertial length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getInertialLength()"
    },

    {
        name      : "Thermal velocity",
        dimension : "velocity",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getThermalVelocity()"
    },
    {
        name      : "Alfven velocity",
        dimension : "velocity",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getAlfvenVelocity(this.B)"
    },

    {
        name      : "Coupling parameter",
        dimension : "none",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getCoupling()"
    },
    {
        name      : "Typical Debye screening",
        dimension : "none",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "var r = %s1.getMeanDistance()/%s1.getDebyeLength(); \
                     (1 + r)*Math.exp(-r)"
    },

    {
        name      : "Transition energy",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "this.getWavenumber()"
    },
    {
        name      : "Natural linewidth",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "this.getNaturalWidth()"
    },
    {
        name      : "Doppler FWHM",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "2*this.getDopplerHwhm()"
    },
    {
        name      : "Zeeman splitting",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "2*this.getZeemanSplitting()"
    },
    {
        name      : "Stark FWHM",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "2*this.getStarkHwhm()"
    },
    {
        name      : "QS Stark FWHM",
        dimension : "energy",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "2*this.getQsHwhm(%s1)"
    },

    {
        name      : "Dynamic Stark range",
        dimension : "energy",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getMicrofieldFrequency(%s1)/3e10/(2*Math.PI)"
    },

    {
        name      : "Minimal energy distance",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "this.getMinEnergyDistance()"
    },

    {
        name      : "Static/dynamic Stark ratio",
        dimension : "none",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getStarkRatio(%s1)"
    },
    {
        name      : "Stark quasistaticity",
        dimension : "none",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getStarkQuasistaticity(%s1)"
    },

    {
        name      : "Transition wavelength",
        dimension : "length",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "1/this.getWavenumber()"
    },

    {
        name      : "Transition frequency",
        dimension : "frequency",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "3e10*this.getWavenumber()"
    },

    {
        name      : "Holtsmark field",
        dimension : "efield",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "%s1.getHoltsmarkField()"
    },

    {
        name      : "Microfield frequency",
        dimension : "frequency",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getMicrofieldFrequency(%s1)"
    }
);


function LymanA(n)
{
    if (n <= 1) {
        return 0;
    } else {
        return 8e9*256/9*n*Math.pow((n - 1)/(n + 1), 2*n)/((n*n - 1)*(n*n - 1));
    }
}


function NumPrint(v, prec)
{
    var s;
    
    if (!prec) {
        prec = 3;
    }
    
    if (v == 0.0) {
        s = "0";
    } else
    if (!isFinite(v)) {
        s = v;
    } else
    if (Math.abs(v) < 1.e6 && Math.abs(v) >= 0.1) {
        s = sprintf("%." + prec.toString() + "f", v);
    } else {
        s = sprintf("%." + prec.toString() + "e", v);
    }
    
    return s;
}

function changeN_e(ev)
{
    var e = ev.getTarget();
    var N_e = e.getNumValue();
    
    if (isFinite(N_e) && this.e.setN(N_e)) {
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeT_e(ev)
{
    var e = ev.getTarget();
    var T_e = e.getNumValue();
    
    if (isFinite(T_e)) {
        this.e.setT(T_e);
        this.i.setT(T_e);
        this.r.setT(T_e);
        this.updateGUI();
    }
}

function changeZ_i(ev)
{
    var e = ev.getTarget();
    var Z_i = parseInt(e.getValue());
    
    if (isFinite(Z_i)) {
        this.i.setQ(Z_i);
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeT_i(ev)
{
    var e = ev.getTarget();
    var T_i = e.getNumValue();
    
    if (isFinite(T_i)) {
        this.i.setT(T_i);
        this.r.setT(T_i);
        this.updateGUI();
    }
}

function changeZ_r(ev)
{
    var e = ev.getTarget();
    var Z_r = parseInt(e.getValue());
    
    if (isFinite(Z_r)) {
        this.r.setQ(Z_r);
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeP_r(ev)
{
    var e = ev.getTarget();
    var P_r = parseInt(e.getValue())/100.0;
    
    if (isFinite(P_r)) {
        this.P_r = P_r;
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeT_r(ev)
{
    var e = ev.getTarget();
    var T_r = e.getNumValue();
    
    if (isFinite(T_r)) {
        this.r.setT(T_r);
        this.updateGUI();
    }
}

function changeM_i(ev)
{
    var e = ev.getTarget();
    var M_i = parseInt(e.getValue());
    
    if (isFinite(M_i)) {
        this.i.setM(M_i);
        this.updateGUI();
    }
}

function changeM_r(ev)
{
    var e = ev.getTarget();
    var M_r = parseInt(e.getValue());
    
    if (isFinite(M_r)) {
        this.r.setM(M_r);
        this.updateGUI();
    }
}

function changeN_u(ev)
{
    var e = ev.getTarget();
    var n_u = parseInt(e.getValue());
    
    this.n_u = n_u;
    this.updateGUI();
}

function changeN_l(ev)
{
    var e = ev.getTarget();
    var n_l = parseInt(e.getValue());
    
    this.n_l = n_l;
    this.updateGUI();
}

function changeB(ev)
{
    var e = ev.getTarget();
    var v = e.getNumValue();
    
    if (isFinite(v)) {
        this.B = v;
        this.updateGUI();
    }
}

function viewMenuCB(ev)
{
    var explorer_is_seen;
    
    if (this.entity_tree.isSeeable()) {
        explorer_is_seen = true;
    } else {
        explorer_is_seen = false;
    }
    
    this.showSectionsButton.setEnabled(explorer_is_seen);
    this.showSpeciesButton.setEnabled(explorer_is_seen);
}

function editMenuCB(ev)
{
    var explorer_is_seen;
    
    if (this.entity_tree.isSeeable()) {
        explorer_is_seen = true;
    } else {
        explorer_is_seen = false;
    }

    var tree = this.entity_tree;
    var leaf = tree.getSelectedElement();

    if (leaf && explorer_is_seen) {
        if (this.isFavorite(leaf)) {
            this.editMenuButtonAddToFav.setEnabled(false);
            this.editMenuButtonRemFromFav.setEnabled(true);
        } else {
            this.editMenuButtonAddToFav.setEnabled(true);
            this.editMenuButtonRemFromFav.setEnabled(false);
        }
    } else {
        this.editMenuButtonAddToFav.setEnabled(false);
        this.editMenuButtonRemFromFav.setEnabled(false);
    }
}

function treeMenuCB(ev)
{
    if (ev.getButton() == "right") {
        var menu = this.treeMenu;

        if (menu.isSeeable()) {
            menu.hide();
        } else {
            var leaf = ev.getTarget();
            var el = leaf.getElement();
            
            if (leaf.classname == "qx.ui.tree.TreeFile") {
                if (this.isFavorite(leaf)) {
                    this.treeMenuButtonAddToFav.setEnabled(false);
                    this.treeMenuButtonRemFromFav.setEnabled(true);
                } else {
                    this.treeMenuButtonAddToFav.setEnabled(true);
                    this.treeMenuButtonRemFromFav.setEnabled(false);
                }
            } else {
                this.treeMenuButtonAddToFav.setEnabled(false);
                this.treeMenuButtonRemFromFav.setEnabled(false);
            }
            
            // position the menu
            menu.setLeft(ev.getClientX());
            menu.setTop(ev.getClientY());

            menu.show();
        };

        ev.setPropagationStopped(true);
    }
}

function showFundamentalCB(ev)
{
    this.showFundamental = !this.showFundamental;
    this.rePopulateTree();
}

function showSpectroscopyCB(ev)
{
    this.showSpectroscopy = !this.showSpectroscopy;
    this.rePopulateTree();
}

function showElectronsCB(ev)
{
    this.showElectrons = !this.showElectrons;
    this.rePopulateTree();
}

function showIonsCB(ev)
{
    this.showIons = !this.showIons;
    this.rePopulateTree();
}

function showRadiatorsCB(ev)
{
    this.showRadiators = !this.showRadiators;
    this.rePopulateTree();
}

function addToFavCB(ev)
{
    var tree = this.entity_tree;
    var leaf = tree.getSelectedElement();

    if (!leaf) {
        return;
    }

    leaf.setTextColor("red");

    // Add to the list of favorites
    var fav = new Array;
    fav.entity  = leaf.getUserData("entity");
    fav.species = leaf.getUserData("species");
    this.favorites.push(fav);
    
    // Update the list of favorites
    this.updateFavoritesListView();
}

function removeFromFavCB(ev)
{
    var tree = this.entity_tree;
    var leaf = tree.getSelectedElement();

    if (!leaf) {
        return;
    }
    leaf.setTextColor("black");

    var entity  = leaf.getUserData("entity");
    var species = leaf.getUserData("species");

    // Remove from the list of favorites
    for (var k = 0; k < this.favorites.length; k++) {
        var fav = this.favorites[k];
        if (fav.entity  == entity && fav.species == species) {
            this.favorites.splice(k, 1);
            break;
        }
    }

    // Update the list of favorites
    this.updateFavoritesListView();
}

function clickEntityCB(ev)
{
    var e = ev.getTarget();
    var entity = e.getUserData("entity");
    
    this.gui_entity_units.update(entity.dimension);
    
    this.updateEntityValue();
}

function changeUnitsCB(ev)
{
    this.updateEntityValue();
}

/**
 * The application
 */
qx.Class.define("pf.Application",
{
    extend : qx.application.Gui,

    /*
    *****************************************************************************
       MEMBERS
    *****************************************************************************
    */

    members :
    {
        R0 : 0.5, // 0.3/0.7/1 ?
        
        showFundamental  : true,
        showSpectroscopy : true,

        showElectrons  : true,
        showIons       : true,
        showRadiators  : true,

        checkPlasmaParameters : function()
        {
            if ((this.P_r < 0.0 || this.P_r > 1.0) ||
                this.Z_i < 1 ||
                this.P_r == 1.0 && this.Z_r == 0) {
                return false;
            } else {
                return true;
            }
        },

        updatePlasmaParameters : function()
        {
            // Safety measures
            if (this.checkPlasmaParameters() != true) {
                return false;
            }
            
            var N_e = this.e.getN();
            var N_i, N_r;
            
            if (this.P_r < 1.0) {
                N_i = N_e*(1.0 - this.P_r)/
                    (this.i.getQ()*(1.0 - this.P_r) + this.r.getQ()*this.P_r);
                
                N_r = N_i*this.P_r/(1.0 - this.P_r);
            } else {
                N_i = 0.0;
                N_r = N_e/this.r.getQ();
            }
            
            this.i.setN(N_i);
            this.r.setN(N_r);
            
            return true;
        },
        
        updateGUI : function()
        {
            this.gui_N_i.setNumValue(this.i.getN());
            this.gui_N_r.setNumValue(this.r.getN());
            this.gui_T_i.setNumValue(this.i.getT());
            this.gui_T_r.setNumValue(this.r.getT());
            this.gui_Zcore.setValue(this.r.getQ() + 1);
            
            // Update the currently selecte entity
            this.updateEntityValue();
            
            // Update the list of favorites
            this.updateFavoritesListView();
        },
        
        getWavenumber : function()
        {
            var n_u = this.n_u;
            var n_l = this.n_l;
            var Z_core = this.r.getQ() + 1;
            
            return pf.base.Bohr.transitionEnergy(Z_core, n_u, n_l);
        },
        
        getMinEnergyDistance : function()
        {
            var n;
            if (this.n_u < this.n_l) {
                n = this.n_l;
            } else {
                n = this.n_u;
            }
            
            var Z_core = this.r.getQ() + 1;
            
            return pf.base.Bohr.transitionEnergy(Z_core, n + 1, n);
        },
        
        getNaturalWidth : function()
        {
            var n_u = this.n_u;
            var n_l = this.n_l;
            var Z_core = this.r.getQ() + 1;
            
            return Math.pow(Z_core, 4)*
                (LymanA(n_u) + LymanA(n_l))/3e10/(2*Math.PI);
        },
        
        getDopplerHwhm : function()
        {
            var v   = this.r.getThermalVelocity();
            var nu  = this.getWavenumber();
            
            return Math.sqrt(2*Math.log(2))*nu*v/3e10;
        },
        
        getMicrofieldFrequency : function(s)
        {
            var r = s.getMeanDistance();
            var v_r = this.r.getThermalVelocity();
            var v_s = s.getThermalVelocity();
            var v = Math.sqrt(v_r*v_r + v_s*v_s);
            
            return v/r;
        },
        
        getFullDebyeLength : function(s)
        {
            var tmp = 0;
            var m = s.getM();
            
            var sa = new Array(this.e, this.i, this.r);
            for (var i = 0; i < 3; i++) {
                var os = sa[i];
                if (os.getM() <= m) {
                    var ld = os.getDebyeLength();
                    if (ld) {
                        tmp += 1/(ld*ld);
                    }
                }
                
            }
            
            return 1/Math.sqrt(tmp);
        },
        
        getPpiFactor : function(s)
        {
            var dl = this.getFullDebyeLength(s);
            var r1 = s.getMeanDistance();
            var r;
            if (isFinite(dl) && isFinite(r1)) {
                r = r1/dl;
            } else {
                r = 0;
            }
            
            return (1 + r)*Math.exp(-r);
            // return 1 - 0.6*r
        },
        
        getRpiFactor : function(s)
        {
            var r1 = s.getMeanDistance();
            var q_r = this.r.getQ();
            var q_s = s.getQ();
            if (q_r == 0 || q_s == 0) {
                return 1;
            }
             
            var r_m = s.getMinApproachDistance()*q_r/q_s;
            return Math.exp(-r_m/r1);
        },
        
        getQsHwhm : function(s)
        {
            var n_u = this.n_u;
            var n_l = this.n_l;
            var Z_core = this.r.getQ() + 1;
            var ef = s.getHoltsmarkField();
            var ppi = this.getPpiFactor(s);
            var rpi = this.getRpiFactor(s);
            
            // 1.4385 is S_1 HWHM
            return 1.4385*4.271417e-5*3/2*(n_u*n_u - n_l*n_l)/Z_core*ef*ppi*rpi;
        },
        
        getStarkRatio : function(s)
        {
            var R;
            var w_qs = this.getQsHwhm(s);
            if (w_qs == 0) {
                R = 0;
            } else {
                R = 2*w_qs/(this.getMicrofieldFrequency(s)/3e10/(2*Math.PI));
            }
            
            return R;
        },
        
        getStarkQuasistaticity : function(s)
        {
            var R = this.getStarkRatio(s);
            return R/(R + this.R0);
        },
        
        getStarkHwhm : function()
        {
            var w = 0;
            
            var sa = new Array(this.e, this.i, this.r);
            for (var i = 0; i < 3; i++) {
                var s = sa[i];
                var qs = this.getQsHwhm(s);
                var f  = this.getStarkQuasistaticity(s);

                w += Math.pow(f*qs, 3/2);
            }
            
            return Math.pow(w, 2/3);
        },
        
        getZeemanSplitting : function()
        {
            return 0.466860*this.B;
        },
        
        getEntityFullName : function (entity, species)
        {
            var name = entity.name;
            if (entity.nspecies == 1) {
                switch (species.id) {
                case "e":
                    name += " of electrons";
                    break;
                case "i":
                    name += " of ions";
                    break;
                case "r":
                    name += " of radiators";
                    break;
                }
            }
            
            return name;
        },
        
        evaluateEntityValue : function(entity, species)
        {
            var formula;
            if (entity.nspecies == 1) {
                var species_str = "this." + species.id;
                formula = entity.formula.replace(/\%s1/g, species_str);
            } else {
                formula = entity.formula;
            }
            var v = eval(formula);
            
            return v;
        },
        
        updateEntityValue : function()
        {
            var tree = this.entity_tree;
            var leaf = tree.getSelectedElement();
            
            if (!leaf) {
                return;
            }
            
            var entity  = leaf.getUserData("entity");
            var species = leaf.getUserData("species");
            
            var name = this.getEntityFullName(entity, species);
            this.gui_entity_name.setValue(name);

            this.gui_entity_section.setValue(entity.section);
            
            var v = this.evaluateEntityValue(entity, species);

            var scale = this.gui_entity_units.getSelected().getValue();

            this.gui_entity_value.setNumValue(v*scale);
        },

        isFavorite : function(leaf)
        {
            var entity  = leaf.getUserData("entity");
            var species = leaf.getUserData("species");

            for (var k = 0; k < this.favorites.length; k++) {
                var fav = this.favorites[k];
                if (fav.entity  == entity && fav.species == species) {
                    return true;
                    break;
                }
            }
            
            return false;
        },

        populateTree : function()
        {
            var i, n = plasma_entities.length;
            var sa = new Array;
            if (this.showElectrons) {
                sa.push(this.e)
            }
            if (this.showIons) {
                sa.push(this.i)
            }
            if (this.showRadiators) {
                sa.push(this.r)
            }
            
            for (var i = 0; i < n; i++) {
                var entity = plasma_entities[i];
                
                if (entity.section == "fundamental" &&
                    this.showFundamental != true) {
                    continue;
                }
                if (entity.section == "spectroscopy" &&
                    this.showSpectroscopy != true) {
                    continue;
                }
                
                var folder;
                switch (entity.dimension) {
                case "frequency":
                    folder = this.gui_folder_freq;
                    break;
                case "length":
                    folder = this.gui_folder_length;
                    break;
                case "velocity":
                    folder = this.gui_folder_vel;
                    break;
                case "energy":
                    folder = this.gui_folder_energy;
                    break;
                case "none":
                    folder = this.gui_folder_none;
                    break;
                default:
                    folder = this.gui_folder_misc;
                    break;
                }
                
                var leaf;
                
                switch (entity.nspecies) {
                case 1:
                    for (var j = 0; j < sa.length; j++) {
                        var s = sa[j];
                        var icon = "pf/image/" + s.id + "_small.png";

                        leaf = new qx.ui.tree.TreeFile(entity.name, icon);
                        leaf.addEventListener("changeSelected", clickEntityCB, this);
                        leaf.addEventListener("contextmenu", treeMenuCB, this);
                        leaf.setUserData("entity", entity);
                        leaf.setUserData("species", s);
                        
                        if (this.isFavorite(leaf)) {
                            leaf.setTextColor("red");
                        }

                        folder.add(leaf);
                    }
                    break;
                default:
                    leaf = new qx.ui.tree.TreeFile(entity.name,
                        "pf/image/all_small.png");
                    leaf.addEventListener("changeSelected", clickEntityCB, this);
                    leaf.addEventListener("contextmenu", treeMenuCB, this);
                    leaf.setUserData("entity", entity);

                    if (this.isFavorite(leaf)) {
                        leaf.setTextColor("red");
                    }
                    
                    folder.add(leaf);
                    break;
                }
            }
        }, 

        cleanTree : function()
        {
            this.gui_folder_freq.destroyContent();
            
            this.gui_folder_length.destroyContent();
            
            this.gui_folder_vel.destroyContent();
            
            this.gui_folder_none.destroyContent();
            
            this.gui_folder_energy.destroyContent();
            
            this.gui_folder_misc.destroyContent();
        },

        rePopulateTree : function()
        {
            // this.entity_tree.hide();

            this.cleanTree();
            this.populateTree();

            this.gui_folder_freq.close();
            this.gui_folder_length.close();
            this.gui_folder_vel.close();
            this.gui_folder_none.close();
            this.gui_folder_energy.close();
            this.gui_folder_misc.close();

            // this.entity_tree.show();
        },


        updateFavoritesListView : function()
        {
            var lv = this.favoritesListView;
            var rowData = lv.getData();
            rowData.length = 0;
            
            var dimension = this.gui_compare_dim.getSelectedValue();
            
            var scale = this.gui_compare_units.getSelectedValue();
            
            for (var i = 0; i < this.favorites.length; i++) {
                var fav = this.favorites[i];
                var entity = fav.entity;
                
                if (entity.dimension != dimension) {
                    continue;
                }
                
                var species = fav.species;

                var icon;
                if (species) {
                    icon = "pf/image/" + species.id + "_small.png";
                } else {
                    icon = "pf/image/all_small.png";
                }
                var name = this.getEntityFullName(entity, species);
                var value = scale*this.evaluateEntityValue(entity, species);

                rowData.push({ s_icon   : {icon       : icon,
                                           iconWidth  : 16, 
                                           iconHeight : 16             },
                               name     : {text       : name           },
                               section  : {text       : entity.section },
                               value    : {text       : NumPrint(value)}
                             });
            }
            
            lv.updateSort();
            lv.update();
        },


        renderTabPage : function(label, icon, checked)
        {
            var tab_id, page_id;
            
            tab_id = new qx.ui.pageview.tabview.Button(label, icon);
            this.tabbar.add(tab_id);
            page_id = new qx.ui.pageview.tabview.Page(tab_id);
            page_id.setHeight("auto");
            page_id.setPadding(10, 10, 10, 10);
            page_id.setBackgroundColor("white");
            this.tabpane.add(page_id);

            if (checked) {
                tab_id.setChecked(true);
            }
            
            return page_id;
        },

        renderParametersTab : function()
        {
            var page_id = this.renderTabPage("Parameters",
                "pf/image/tab-parameters.png", true);

            var fr, gl;
            
            fr = new qx.ui.groupbox.GroupBox("Electrons",
                "pf/image/e_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#eeeeff");

            page_id.add(fr);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 260);
            gl.setRowCount(2);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);

            fr.add(gl);

            var l, e;

            // N_e
            l = new qx.ui.basic.Label("N<sub>e</sub> (cm<sup>-3</sup>):");
            gl.add(l, 0, 0);
            e = new pf.ui.TextField(this.e.getN());
            gl.add(e, 1, 0);
            // e.setLiveUpdate(true);
            e.addEventListener("changeValue", changeN_e, this);

            // T_e
            l = new qx.ui.basic.Label("T<sub>e</sub> (eV):");
            gl.add(l, 0, 1);
            e = new pf.ui.TextField(this.e.getT());
            gl.add(e, 1, 1);
            e.addEventListener("changeValue", changeT_e, this);
            e.setLiveUpdate(true);


            fr = new qx.ui.groupbox.GroupBox("Ions",
                "pf/image/i_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#ffeeee");

            page_id.add(fr);
            fr.setTop(100);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 260);
            gl.setRowCount(4);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);
            gl.setRowHeight(3, 30);

            fr.add(gl);

            // Z_i
            l = new qx.ui.basic.Label("Z<sub>i</sub>:");
            gl.add(l, 0, 0);
            e = new pf.ui.Spinner(1, this.i.getQ(), 100);
            gl.add(e, 1, 0);
            
            e.addEventListener("change", changeZ_i, this);
            
            // M_i
            l = new qx.ui.basic.Label("M<sub>i</sub>:");
            gl.add(l, 0, 1);
            e = new pf.ui.Spinner(1, this.i.getM(), 200);
            gl.add(e, 1, 1);
            e.addEventListener("change", changeM_i, this);
            
            // T_i
            l = new qx.ui.basic.Label("T<sub>i</sub> (eV):");
            gl.add(l, 0, 2);
            e = new pf.ui.TextField(this.i.getT());
            gl.add(e, 1, 2);
            e.addEventListener("changeValue", changeT_i, this);
            e.setLiveUpdate(true);
            this.gui_T_i = e;

            // N_i
            l = new qx.ui.basic.Label("N<sub>i</sub> (cm<sup>-3</sup>):");
            gl.add(l, 0, 3);
            e = new pf.ui.TextField(this.i.getN());
            gl.add(e, 1, 3);
            e.setReadOnly(true);
            this.gui_N_i = e;
            
            
            fr = new qx.ui.groupbox.GroupBox("Radiators",
                "pf/image/r_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#eeffee");

            page_id.add(fr);
            fr.setLeft(390);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 260);
            gl.setRowCount(5);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);
            gl.setRowHeight(3, 30);
            gl.setRowHeight(4, 30);

            fr.add(gl);


            // Z_r
            l = new qx.ui.basic.Label("Z<sub>r</sub>:");
            gl.add(l, 0, 0);
            e = new pf.ui.Spinner(0, this.Z_r, 100);
            gl.add(e, 1, 0);
            e.addEventListener("change", changeZ_r, this);

            // M_r
            l = new qx.ui.basic.Label("M<sub>r</sub>:");
            gl.add(l, 0, 1);
            e = new pf.ui.Spinner(1, this.r.getM(), 200);
            gl.add(e, 1, 1);
            e.addEventListener("change", changeM_r, this);

            // T_r
            l = new qx.ui.basic.Label("T<sub>r</sub> (eV):");
            gl.add(l, 0, 2);
            e = new pf.ui.TextField(this.r.getT());
            gl.add(e, 1, 2);
            e.addEventListener("changeValue", changeT_r, this);
            this.gui_T_r = e;

            // P_r
            l = new qx.ui.basic.Label("P<sub>r</sub> (%):");
            gl.add(l, 0, 3);
            e = new pf.ui.Spinner(0, this.P_r, 100);
            gl.add(e, 1, 3);
            e.addEventListener("change", changeP_r, this);

            // N_r
            l = new qx.ui.basic.Label("N<sub>r</sub> (cm<sup>-3</sup>):");
            gl.add(l, 0, 4);
            e = new pf.ui.TextField(this.r.getN());
            e.setReadOnly(true);
            gl.add(e, 1, 4);
            this.gui_N_r = e;


            fr = new qx.ui.groupbox.GroupBox("Fields",
                "pf/image/field_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#ffffdd");

            page_id.add(fr);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 260);
            gl.setRowCount(1);
            gl.setRowHeight(0, 30);

            fr.add(gl);
            fr.setTop(268);


            l = new qx.ui.basic.Label("B (T):");
            gl.add(l, 0, 0);
            e = new pf.ui.TextField(this.B);
            gl.add(e, 1, 0);
            // e.setLiveUpdate(true);
            e.addEventListener("changeValue", changeB, this);

            // Atomic system frame
            fr = new qx.ui.groupbox.GroupBox("Atomic system",
                "pf/image/all_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#ffffdd");

            page_id.add(fr);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 260);
            gl.setRowCount(3);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);

            fr.add(gl);
            fr.setLeft(390);
            fr.setTop(202);

            l = new qx.ui.basic.Label("Z<sub>core</sub>:");
            gl.add(l, 0, 0);
            e = new qx.ui.form.TextField(this.r.getQ() + 1);
            e.setReadOnly(true);
            gl.add(e, 1, 0);
            this.gui_Zcore = e;

            l = new qx.ui.basic.Label("n<sub>u</sub>:");
            gl.add(l, 0, 1);
            e = new pf.ui.Spinner(2, this.n_u, 1000);
            gl.add(e, 1, 1);
            e.addEventListener("change", changeN_u, this);

            l = new qx.ui.basic.Label("n<sub>l</sub>:");
            gl.add(l, 0, 2);
            e = new pf.ui.Spinner(1, this.n_l, 1000);
            gl.add(e, 1, 2);
            e.addEventListener("change", changeN_l, this);
        },

        renderExplorerTab : function()
        {
            var page_id = this.renderTabPage("Explorer",
                "pf/image/tab-explorer.png");
            
            var sp = new qx.ui.splitpane.HorizontalSplitPane(300, "1*");
            sp.set(
            {
                height          : "100%",
                left            : 1,
                right           : 1,
                top             : 1,
                bottom          : 1,
                border          : "inset-thin",
                backgroundColor : "white",
                showKnob        : true
            });
            page_id.add(sp);
                        
            var left_pane  = sp.getLeftArea();
            left_pane.setPaddingRight(4);
            var right_pane = sp.getRightArea();
            right_pane.setPaddingLeft(4);
            var tree = new qx.ui.tree.Tree("Browser");
            tree.set(
            {
                border          : "inset-thin",
                overflow        : "scroll",
                height          : "100%",
                width           : "100%",
                paddingLeft     : 4,
                paddingTop      : 4
            });
            // tree.addEventListener("contextmenu", treeMenuCB, this);
            left_pane.add(tree);
            
            this.entity_tree = tree;
            
            var folder;
            
            folder = new qx.ui.tree.TreeFolder("Frequencies");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_freq = folder;

            folder = new qx.ui.tree.TreeFolder("Lengths");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_length = folder;
            
            folder = new qx.ui.tree.TreeFolder("Velocities");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_vel = folder;
            
            folder = new qx.ui.tree.TreeFolder("Dimensionless");
            tree.add(folder);
            this.gui_folder_none = folder;

            folder = new qx.ui.tree.TreeFolder("Energies");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_energy = folder;

            folder = new qx.ui.tree.TreeFolder("Miscellaneous");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_misc = folder;

            this.populateTree();

            tree.setHideNode(true);
            tree.setRootOpenClose(true);
            
            // Build context-sensitive popup menu
            this.treeMenu = new qx.ui.menu.Menu;
            var button;
            
            button = new qx.ui.menu.Button("Add to Favorites",
                "pf/image/favorite-add.png");
            button.addEventListener("execute", addToFavCB, this);
            this.treeMenu.add(button);
            this.treeMenuButtonAddToFav = button;
            button = new qx.ui.menu.Button("Remove from Favorites",
                "pf/image/favorite-delete.png");
            button.addEventListener("execute", removeFromFavCB, this);
            this.treeMenu.add(button);
            this.treeMenuButtonRemFromFav = button;
            
            var doc = qx.ui.core.ClientDocument.getInstance();
            doc.add(this.treeMenu);


            var gl = new qx.ui.layout.GridLayout;
            gl.set(
            {
                left:   4,
                right:  4,
                top:    4
            });
            
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(3);
            gl.setColumnWidth(0, "15%");
            gl.setColumnWidth(1, "60%");
            gl.setColumnWidth(2, "20%");
            gl.setRowCount(3);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);

            var l, e;

            l = new qx.ui.basic.Label("Name:");
            gl.add(l, 0, 0);
            e = new qx.ui.form.TextField();
            e.setReadOnly(true);
            this.gui_entity_name = e;
            gl.add(e, 1, 0);

            l = new qx.ui.basic.Label("Value:");
            gl.add(l, 0, 1);
            e = new pf.ui.TextField("");
            e.setReadOnly(true);
            gl.add(e, 1, 1);
            this.gui_entity_value = e;

            e = new pf.UnitSelector();
            e.addEventListener("changeSelected", changeUnitsCB, this);
            gl.add(e, 2, 1);
            this.gui_entity_units = e;

            l = new qx.ui.basic.Label("Section:");
            gl.add(l, 0, 2);
            e = new qx.ui.form.TextField();
            e.setReadOnly(true);
            this.gui_entity_section = e;
            gl.add(e, 1, 2);

            right_pane.add(gl);
        },
        
        renderCompareTab : function()
        {
            var page_id = this.renderTabPage("Compare",
                "pf/image/tab-compare.png");
            
            var bl = new qx.ui.layout.VerticalBoxLayout("vertical");
            bl.setWidth("100%");
            bl.setHeight("100%");
            bl.setSpacing(10);
            page_id.add(bl);
            
            var rc = new qx.ui.layout.HorizontalBoxLayout;
            rc.setWidth("100%");
            rc.setHeight("auto");
            rc.setSpacing(10);
            bl.add(rc);

            var l, e;
            l = new qx.ui.basic.Label("Dimension:");
            e = new pf.ui.ComboBox;
            e.add(new qx.ui.form.ListItem("Frequency", null, "frequency"));
            e.add(new qx.ui.form.ListItem("Length", null, "length"));
            e.add(new qx.ui.form.ListItem("Velocity", null, "velocity"));
            e.add(new qx.ui.form.ListItem("Energy", null, "energy"));
            e.add(new qx.ui.form.ListItem("Electric field", null, "efield"));
            e.add(new qx.ui.form.ListItem("None", null, "none"));
            e.addEventListener("changeSelected", function(ev) {
                    var dimension = this.gui_compare_dim.getSelectedValue();
                    this.gui_compare_units.update(dimension);
                    this.updateFavoritesListView();
                }, this);
            rc.add(l, e);
            this.gui_compare_dim = e;

            l = new qx.ui.basic.Label("Units:");
            e = new pf.UnitSelector;
            e.addEventListener("changeSelected", function(ev) {
                    this.updateFavoritesListView();
                }, this);
            rc.add(l, e);
            this.gui_compare_units = e;

            var lc =
            {
                s_icon   : { label      : "",
                             width      : 24,
                             type       : "iconHtml",
                             align      : "center",                    
                             sortable   : false                         },
                name     : { label      : "Name",
                             width      : 300,
                             type       : "text",
                             align      : "left",
                             sortable   : true,
                             sortProp   : "text"                        },
                section  : { label      : "Section",
                             width      : 120,
                             type       : "text",
                             align      : "left",
                             sortable   : true,
                             sortProp   : "text"                        },
                value    : { label      : "Value",
                             width      : 120,
                             type       : "text",
                             align      : "right",
                             sortable   : true,
                             sortProp   : "text",
                             sortMethod : qx.util.Compare.byFloatString }
            };

            var lv = new qx.ui.listview.ListView(new Array, lc);

            lv.setWidth("100%");
            lv.setHeight(450);
            lv.setBackgroundColor("white");
            lv.setBorder("dark-shadow");

            bl.add(lv);
            
            this.favoritesListView = lv;
        },
        
        drawCB : function(ev)
        {
            var canvas = this.canvas;
            
            var pname = this.guiParameter.getValue();
            var vmin = this.guiMinParam.getNumValue();
            var vmax = this.guiMaxParam.getNumValue();
            var npoints = this.guiParamPoints.getValue();
            
            if (npoints < 2 || !pname) {
                return;
            }

            var dimension = this.gui_compare_dim.getSelectedValue();
            var scale = this.gui_compare_units.getSelectedValue();

            var par0;
            switch (pname) {
            case "N_e":
                par0 = this.e.getN();
                break;
            case "T_e":
            case "T":
                par0 = this.e.getT();
                break;
            case "T_i":
                par0 = this.i.getT();
                break;
            case "B":
                par0 = this.B;
                break;
            default:
                return;
            }
            
            var xdata = new Array;
            for (var j = 0; j < npoints; j++) {
                var x = par0*(vmin + (vmax - vmin)*j/(npoints - 1));
                xdata.push(x);
            }
                        
            var wxmin = xdata[0];
            var wxmax = xdata[xdata.length - 1];

            var wymin = 0;
            var wymax = 0;

            var sets = new Array;
            var first = true;
            for (var i = 0; i < this.favorites.length; i++) {
                var fav = this.favorites[i];
                var entity = fav.entity;
                
                if (entity.dimension != dimension) {
                    continue;
                }
                
                var species = fav.species;
                var vname = this.getEntityFullName(entity, species);

                var set   = new Array;
                var ydata = new Array;
                for (var j = 0; j < npoints; j++) {
                    var par = xdata[j];

                    switch (pname) {
                    case "N_e":
                        this.e.setN(par);
                        this.updatePlasmaParameters();
                        break;
                    case "T_e":
                        this.e.setT(par);
                        break;
                    case "T_i":
                        this.i.setT(par);
                        break;
                    case "T":
                        this.e.setT(par);
                        this.i.setT(par);
                        this.r.setT(par);
                        break;
                    case "B":
                        this.B = par;
                        break;
                    }
                    
                    var value = scale*this.evaluateEntityValue(entity, species);
                    
                    if (first) {
                        wymin = value;
                        wymax = value;
                        first = false;
                    } else {
                        if (value < wymin) {
                            wymin = value;
                        }
                        if (value > wymax) {
                            wymax = value;
                        }
                    }
                    
                    ydata.push(value);
                }
                set.x = xdata;
                set.y = ydata;
                set.legend = vname;
                sets.push(set);
            }
            
            // Restore parameter
            switch (pname) {
            case "N_e":
                this.e.setN(par0);
                this.updatePlasmaParameters();
                break;
            case "T_e":
                this.e.setT(par0);
                break;
            case "T_i":
                this.i.setT(par0);
                break;
            case "T":
                this.e.setT(par0);
                this.i.setT(par0);
                this.r.setT(par0);
                break;
            case "B":
                this.B = par0;
                break;
            }
            
            canvas.clear();
            
            canvas.setViewportCoordinates(0.15, 0.1, 0.9, 0.9);
            dwx = Math.abs(wxmax - wxmin);
            if (dwx == 0) {
                dwx = wxmax;
            }
            dwy = Math.abs(wymax - wymin);
            if (dwy == 0) {
                wymin = 0.0;
                wymax = 1.0;
            }
            canvas.setWorldCoordinates(Math.max(0, wxmin - 0.05*dwx),
                Math.max(0, wymin - 0.05*dwy),
                wxmax + 0.05*dwx, wymax + 0.05*dwy);
            
            canvas.drawFrame("black");
            canvas.drawTicks("x", "blue");
            canvas.drawTicks("y", "blue");
            
            canvas.setColor("red");
            for (var i = 0; i < sets.length; i++) {
                var set = sets[i];
                canvas.drawPolyLineW(set.x, set.y);
                
                delete set;
            }
            
            delete sets;
            
            canvas.paint();
        },
        
        renderPlotTab : function()
        {
            var page_id = this.renderTabPage("Plot",
                "pf/image/tab-plot.png");

            var sp = new qx.ui.splitpane.HorizontalSplitPane("1*", 300);
            sp.set(
            {
                height          : "100%",
                width           : "100%",
                left            : 1,
                right           : 1,
                top             : 1,
                bottom          : 1,
                border          : "inset-thin",
                showKnob        : true
            });
            page_id.add(sp);
                        
            var left_pane  = sp.getLeftArea();
            left_pane.setPaddingRight(4);
            var right_pane = sp.getRightArea();
            right_pane.setPaddingLeft(4);

            var canvas = new pf.ui.canvas.Plotter();
            canvas.setBackgroundColor("white");
            canvas.setWidth("100%");
            canvas.setHeight("100%");
            left_pane.add(canvas);
            this.canvas = canvas;


            var gl = new qx.ui.layout.GridLayout;
            gl.set(
            {
                left:   4,
                right:  4,
                top:    4
            });
            
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 150);
            gl.setColumnWidth(1, 100);
            gl.setRowCount(5);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);
            gl.setRowHeight(3, 30);
            gl.setRowHeight(4, 30);
            right_pane.add(gl);

            var l, e;

            var btn = new qx.ui.form.Button("Redraw");
            btn.addEventListener("execute", this.drawCB, this);
            btn.setTop(20);
            btn.setLeft(20);
            gl.add(btn, 0, 0);

            l = new qx.ui.basic.Label("Parameter to vary:");
            e = new pf.ui.ComboBox;
            e.add(new qx.ui.form.ListItem("N_e", null, "N_e"));
            e.add(new qx.ui.form.ListItem("T",   null, "T"));
            e.add(new qx.ui.form.ListItem("T_e", null, "T_e"));
            e.add(new qx.ui.form.ListItem("T_i", null, "T_i"));
            e.add(new qx.ui.form.ListItem("B",   null, "B"));
            this.guiParameter = e;
            gl.add(l, 0, 1);
            gl.add(e, 1, 1);

            l = new qx.ui.basic.Label("Minimum scale:");
            e = new pf.ui.TextField(0.3);
            this.guiMinParam = e;
            gl.add(l, 0, 2);
            gl.add(e, 1, 2);

            l = new qx.ui.basic.Label("Maximum scale:");
            e = new pf.ui.TextField(3.0);
            this.guiMaxParam = e;
            gl.add(l, 0, 3);
            gl.add(e, 1, 3);

            l = new qx.ui.basic.Label("Number of points:");
            e = new pf.ui.Spinner(2, 10, 100);
            this.guiParamPoints = e;
            gl.add(l, 0, 4);
            gl.add(e, 1, 4);
        },


        aboutCB : function(ev)
        {
            this.aboutWindow.open();
        },
        
        buildMenu : function()
        {
            var menubar = this.menubar;
            var doc = qx.ui.core.ClientDocument.getInstance();
            
            var submenu, btn;

            var bg = "#efebec";
            
            submenu = new qx.ui.menu.Menu;
            submenu.setBackgroundColor(bg);
            doc.add(submenu);
            btn = new qx.ui.menu.Button("Open...", "pf/image/open.png");
            btn.addEventListener("execute", function (ev) {
                alert("Not implemented yet")}, this);
            submenu.add(btn);
            btn = new qx.ui.menu.Button("Save...", "pf/image/save.png");
            btn.addEventListener("execute", function (ev) {
                alert("Not implemented yet")}, this);
            submenu.add(btn);
            submenu.add(new qx.ui.menu.Separator);
            btn = new qx.ui.menu.Button("Exit", "pf/image/exit.png");
            btn.addEventListener("execute", function (ev) {
                window.close()}, this);
            submenu.add(btn);
                
            btn = new qx.ui.menubar.Button("File", submenu);
            btn.set({ paddingLeft : 7, paddingRight : 7 });
            menubar.add(btn);


            submenu = new qx.ui.menu.Menu;
            submenu.setBackgroundColor(bg);
            submenu.addEventListener("beforeAppear", editMenuCB, this);
            doc.add(submenu);
            btn = new qx.ui.menu.Button("Add to Favorites",
                "pf/image/favorite-add.png");
            btn.addEventListener("execute", addToFavCB, this);
            this.editMenuButtonAddToFav = btn;
            submenu.add(btn);
            btn = new qx.ui.menu.Button("Remove from Favorites",
                "pf/image/favorite-delete.png");
            btn.addEventListener("execute", removeFromFavCB, this);
            this.editMenuButtonRemFromFav = btn;
            submenu.add(btn);
            submenu.add(new qx.ui.menu.Separator);
            btn = new qx.ui.menu.Button("Preferences",
                "pf/image/preferences.png");
            btn.addEventListener("execute", function (ev) {
                alert("Not implemented yet")}, this);
            submenu.add(btn);

            btn = new qx.ui.menubar.Button("Edit", submenu);
            btn.set({ paddingLeft : 7, paddingRight : 7 });
            menubar.add(btn);


            submenu = new qx.ui.menu.Menu;
            submenu.setBackgroundColor(bg);
            submenu.addEventListener("beforeAppear", viewMenuCB, this);

            var subMenu1 = new qx.ui.menu.Menu;
            subMenu1.setBackgroundColor(bg);
            button = new qx.ui.menu.CheckBox("Fundamental",
                null, this.showFundamental);
            button.addEventListener("execute", showFundamentalCB, this);
            subMenu1.add(button);
            button = new qx.ui.menu.CheckBox("Spectroscopy",
                null, this.showSpectroscopy);
            button.addEventListener("execute", showSpectroscopyCB, this);
            subMenu1.add(button);
            this.showSectionsButton = new qx.ui.menu.Button("Show Sections",
                null, null, subMenu1);
            submenu.add(this.showSectionsButton);
            
            submenu.add(new qx.ui.menu.Separator);

            var subMenu2 = new qx.ui.menu.Menu;
            subMenu2.setBackgroundColor(bg);
            button = new qx.ui.menu.CheckBox("Electrons",
                null, this.showElectrons)
            button.addEventListener("execute", showElectronsCB, this);
            subMenu2.add(button);
            button = new qx.ui.menu.CheckBox("Ions",
                null, this.showIons)
            button.addEventListener("execute", showIonsCB, this);
            subMenu2.add(button);
            button = new qx.ui.menu.CheckBox("Radiators",
                null, this.showRadiators)
            button.addEventListener("execute", showRadiatorsCB, this);
            subMenu2.add(button);
            this.showSpeciesButton = new qx.ui.menu.Button("Show Species",
                null, null, subMenu2);
            submenu.add(this.showSpeciesButton);
            
            doc.add(submenu, subMenu1, subMenu2);

            btn = new qx.ui.menubar.Button("View", submenu);
            btn.set({ paddingLeft : 7, paddingRight : 7 });
            menubar.add(btn);


            submenu = new qx.ui.menu.Menu;
            submenu.setBackgroundColor(bg);
            doc.add(submenu);
            btn = new qx.ui.menu.Button("Help Contents",
                "pf/image/help.png");
            btn.addEventListener("execute", function (ev) {
                alert("Not implemented yet")}, this);
            submenu.add(btn);
            btn = new qx.ui.menu.Button("NRL Plasma Formulary",
                "pf/image/nrl.png");
            btn.addEventListener("execute", function (ev) {
                    var w = new qx.client.NativeWindow("http://wwwppd.nrl.navy.mil/nrlformulary/");
                    w.setDimension(800, 600);
                    w.open();
                }, this);
            submenu.add(btn);
            submenu.add(new qx.ui.menu.Separator);
            btn = new qx.ui.menu.Button("About Plasma Formulary",
                "pf/image/about.png");
            btn.addEventListener("execute", this.aboutCB, this);
            submenu.add(btn);

            btn = new qx.ui.menubar.Button("Help", submenu);
            btn.set({ paddingLeft : 7, paddingRight : 7 });
            menubar.add(btn);
        },

        /**
         * TODOC
         *
         * @type member
         * @param e {Event} TODOC
         * @return {void}
         */
        main : function(e)
        {
            this.base(arguments);
            
            // Define alias for custom resource path
            qx.io.Alias.getInstance().add("pf",
                qx.core.Setting.get("pf.resourceUri"));

            this.doc = qx.ui.core.ClientDocument.getInstance();
            
            // Defaults
            
            this.r = new pf.base.Species("r", 1.0,           0, 0,    1.0, null);
            this.e = new pf.base.Species("e", (1.0/1836.2), -1, 1e16, 1.0, this.r);
            this.i = new pf.base.Species("i", 1.0,          +1, 1e16, 1.0, this.r);
            
            this.P_r = 0;

            this.B   = 0;
            
            this.n_u = 2;
            this.n_l = 1;
            
            if (this.updatePlasmaParameters() != true) {
                alert("Error!");
            }
            
            // Favorites
            this.favorites = new Array;

            // Top-level container
            this.bl_top = new qx.ui.layout.VerticalBoxLayout("vertical");
            this.bl_top.setWidth("100%");
            this.bl_top.setHeight("100%");

            this.bl_top.addToDocument();

            // The menubar
            this.menubar = new qx.ui.menubar.MenuBar;
            this.bl_top.add(this.menubar);
            
            // The menu items
            this.buildMenu();
            
            var tf1 = new qx.ui.pageview.tabview.TabView;
            tf1.set({ width : '100%', height : 'auto' });

            this.bl_top.add(tf1);

            this.tabbar  = tf1.getBar();
            this.tabpane = tf1.getPane();
            this.tabpane.setHeight('auto');
            this.tabpane.setPadding(0, 0, 0, 0);

            // The "Parameters" tab
            this.renderParametersTab();
            
            // The "Explorer" tab
            this.renderExplorerTab();
            
            // The "Compare" tab
            this.renderCompareTab();
            
            // The "Plot" tab
            this.renderPlotTab();

            // "About..." dialog
            var w = new qx.ui.window.Window("About Plasma Formulary",
                "pf/image/about.png");
            w.setSpace(200, "auto", 100, "auto");
            w.setModal(true);
            w.setShowMinimize(false);
            w.setShowMaximize(false);
            
            str = 'Plasma Formulary Interactive.<br/>'      +
                  '$Revision$.'                      +
                  '<p>Written by Evgeny Stambulchik.</p>'   +
                  '<p>Using the Qooxdoo-' + qx.core.Version + 
                  ' toolkit.</p>';

            var a = new qx.ui.basic.Atom(str);
            a.set({ top: 4, left: 4 });
            w.add(a);

            this.doc.add(w);
            this.aboutWindow = w;
            


        },
    

        /**
         * TODOC
         *
         * @type member
         * @param e {Event} TODOC
         * @return {void}
         */
        close : function(e)
        {
            this.base(arguments);

            // Prompt user
            // return "qooxdoo application: Do you really want to close the application?";
        },


       /**
        * TODOC
        *
        * @type member
        * @param e {Event} TODOC
        * @return {void}
        */
        terminate : function(e) {
            this.base(arguments);
        }
    },

    /*
    *****************************************************************************
       SETTINGS
    *****************************************************************************
    */

    settings : {
        "pf.resourceUri" : "./resource"
    }
});
